# -*- coding: utf-8 -*-
"""
    Colubrid Exceptions
    ===================

    Since paste covers that too this is only a "redirection module".
    Because colubrid may change the error interface later it's
    better to use the mapped names instead of the paste.httpexceptions
    module.
"""
from colubrid.response import HttpResponse
from colubrid.utils import get_version

__all__ = ['PageNotFound', 'PageGone', 'AccessDenied', 'BadRequest',
           'RequestTimeout', 'ServerError', 'HttpRedirect', 'HttpFound',
           'HttpMoved']


ERROR_PAGE_TEMPLATE = """\
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html>
 <head>
  <title>%(code)s %(title)s</title>
  <style type="text/css">
    body {
        font-family: sans-serif;
        margin: 2em;
        padding: 0;
    }
    a, h1 {
        color: #cc0000;
    }
    div.content {
        margin: 1em 3em 2em 2em;
    }
    address {
        border-top: 1px solid #ccc;
        padding: 0.3em;
    }
  </style>
 </head>
 <body>
<h1>%(title)s</h1>
<div class="content">%(msg)s</div>
<address>powered by colubrid %(version)s</address>
</body></html>
"""


class HttpException(Exception):
    code = None
    title = None
    msg = ''
    headers = []

    def __repr__(self):
        return '<%s %d>' % (self.__class__.__name__, self.code)


class HttpMove(HttpException):
    msg = 'The resource has been moved to %s.'
    
    def __init__(self, url):
        headers = [('Location', url)]
        self.headers += headers
        if '%s' in self.msg:
            self.msg = self.msg % url


class PageNotFound(HttpException):
    code = 404
    title = 'Not Found'
    msg = 'The resource could not be found.'


class PageGone(HttpException):
    code = 410
    title = 'Gone'
    msg = 'This resource is no longer available. No forwarding address is given.'


class AccessDenied(HttpException):
    code = 403
    title = 'Forbidden'
    msg = 'Access was denied to this resource.'


class BadRequest(HttpException):
    code = 400
    title = 'Bad Request'
    msg = 'The server could not comply with the request since it is either '\
          'malformed or wtherwise incorrect.'


class RequestTimeout(HttpException):
    code = 408
    title = 'Request Timeout'
    msg = 'There was a conflict when trying to complete your request.'


class ServerError(HttpException):
    code = 500
    title = 'Internal Server Error'
    msg = 'The server has either erred or is inapable of performing '\
          'the requested operation.'


class HttpRedirect(HttpMove):
    code = 307
    title = 'Temporary Redirect'


class HttpFound(HttpMove):
    code = 302
    title = 'Found'
    msg = 'The resource was found at %s.'


class HttpMoved(HttpMove):
    code = 301
    title = 'Moved Permanently'
